/// <reference types="backbone" />
import { WidgetModel, IWidgetManager } from '@jupyter-widgets/base';
import { ScaleSequential, ScaleQuantize, ScaleQuantile, ScaleOrdinal } from 'd3-scale';
export interface ISerializerMap {
    [key: string]: {
        deserialize?: (value?: any, manager?: IWidgetManager) => any;
        serialize?: (value?: any, widget?: WidgetModel) => any;
    };
}
export interface IInitializeOptions {
    model_id: string;
    comm?: any;
    widget_manager: IWidgetManager;
}
/**
 * Base model for scales
 */
export declare abstract class ScaleModel extends WidgetModel {
    /**
     * Returns default values for the model attributes.
     */
    defaults(): {
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    /**
     * Backbone initialize function.
     */
    initialize(attributes: Backbone.ObjectHash, options: IInitializeOptions): void;
    createPropertiesArrays(): void;
    /**
     * Update the model attributes from the objects properties.
     *
     * The base method calls `this.set(toSet, 'pushFromObject');`.
     * Overriding methods should add any properties not in
     * simpleProperties to the hash before calling the super
     * method.
     */
    syncToModel(toSet: Backbone.ObjectHash): void;
    /**
     * Update the model attributes from the objects properties.
     */
    syncToObject(): void;
    /**
     * Create or return the underlying object this model represents.
     */
    protected createObject(): Promise<any>;
    /**
     * Construct and return the underlying object this model represents.
     *
     * Override this in inherting classes.
     */
    protected abstract constructObject(): any | Promise<any>;
    /**
     * Process a new underlying object to represent this model.
     *
     * The base implementation sets up mapping between the model,
     * the cache, and the widget manager.
     */
    protected processNewObj(obj: any): any | Promise<any>;
    /**
     * Set up any event listeners.
     *
     * Called after object initialization is complete.
     */
    setupListeners(): void;
    onChange(model: WidgetModel, options: any): void;
    onChildChanged(model: WidgetModel, options: any): void;
    onCustomMessage(content: any, buffers: any): void;
    static serializers: ISerializerMap;
    static model_name: string;
    static model_module: any;
    static model_module_version: any;
    static view_name: null;
    static view_module: null;
    static view_module_version: any;
    /**
     * The underlying object this model represents.
     */
    obj: any;
    /**
     * Promise that resolves when initialization is complete.
     */
    initPromise: Promise<void>;
    datawidgetProperties: string[];
    childModelProperties: string[];
    simpleProperties: string[];
}
/**
 * A widget model of a sequential scale
 */
export declare abstract class SequentialScaleModel<Output> extends ScaleModel {
    defaults(): {
        domain: number[];
        clamp: boolean;
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    obj: ScaleSequential<Output>;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: WidgetModel | undefined) => any) | undefined;
        };
    };
}
/**
 * A widget model of a quantize scale
 */
export declare class QuantizeScaleModel extends ScaleModel {
    defaults(): {
        domain: number[];
        range: number[];
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    constructObject(): ScaleQuantize<any>;
    obj: ScaleQuantize<any>;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: WidgetModel | undefined) => any) | undefined;
        };
    };
    static model_name: string;
}
/**
 * A widget model of a quantile scale
 */
export declare class QuantileScaleModel extends ScaleModel {
    defaults(): {
        domain: number[];
        range: number[];
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    constructObject(): ScaleQuantile<any>;
    obj: ScaleQuantile<any>;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: WidgetModel | undefined) => any) | undefined;
        };
    };
    static model_name: string;
}
/**
 * A widget model of a teshold scale
 */
export declare class TresholdScaleModel extends ScaleModel {
    defaults(): {
        domain: never[];
        range: number[];
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    constructObject(): ScaleQuantile<any>;
    obj: ScaleQuantile<any>;
    static serializers: {
        [x: string]: {
            deserialize?: ((value?: any, manager?: IWidgetManager | undefined) => any) | undefined;
            serialize?: ((value?: any, widget?: WidgetModel | undefined) => any) | undefined;
        };
    };
    static model_name: string;
}
/**
 * A widget model of an ordinal scale
 */
export declare class OrdinalScaleModel extends ScaleModel {
    defaults(): {
        domain: never[];
        range: never[];
        unknown: {
            name: "implicit";
        };
        _model_name: any;
        _model_module: any;
        _model_module_version: any;
        _view_name: any;
        _view_module: any;
        _view_module_version: any;
    };
    createPropertiesArrays(): void;
    constructObject(): ScaleOrdinal<string, unknown>;
    /**
     * Update the model attributes from the objects properties.
     */
    syncToModel(toSet: Backbone.ObjectHash): void;
    /**
     * Update the model attributes from the objects properties.
     */
    syncToObject(): void;
    obj: ScaleOrdinal<any, any>;
    static serializers: {
        unknown: {
            deserialize: (value?: any, manager?: IWidgetManager) => any;
            serialize: (value?: any, widget?: WidgetModel) => any;
        };
    };
    static model_name: string;
}
