"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.OrdinalScaleModel = exports.TresholdScaleModel = exports.QuantileScaleModel = exports.QuantizeScaleModel = exports.SequentialScaleModel = exports.ScaleModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const d3_scale_1 = require("d3-scale");
const jupyter_dataserializers_1 = require("jupyter-dataserializers");
const version_1 = require("./version");
/**
 * Base model for scales
 */
class ScaleModel extends base_1.WidgetModel {
    /**
     * Returns default values for the model attributes.
     */
    defaults() {
        const ctor = this.constructor;
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ctor.model_name, _model_module: ctor.model_module, _model_module_version: ctor.model_module_version, _view_name: ctor.view_name, _view_module: ctor.view_module, _view_module_version: ctor.view_module_version });
    }
    /**
     * Backbone initialize function.
     */
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.createPropertiesArrays();
        // Instantiate scale object
        this.initPromise = this.createObject().then(() => {
            // sync the properties from the server to the model
            this.syncToObject();
            // sync any properties that might have mutated back
            this.syncToModel({});
            // setup msg, model, and children change listeners
            this.setupListeners();
        });
    }
    createPropertiesArrays() {
        this.datawidgetProperties = [];
        this.childModelProperties = [];
        this.simpleProperties = [];
    }
    /**
     * Update the model attributes from the objects properties.
     *
     * The base method calls `this.set(toSet, 'pushFromObject');`.
     * Overriding methods should add any properties not in
     * simpleProperties to the hash before calling the super
     * method.
     */
    syncToModel(toSet) {
        for (let name of this.simpleProperties) {
            toSet[name] = this.obj[name]();
        }
        // Apply all direct changes at once
        this.set(toSet, 'pushFromObject');
    }
    /**
     * Update the model attributes from the objects properties.
     */
    syncToObject() {
        // Sync the simple properties:
        for (let name of this.simpleProperties) {
            this.obj[name](this.get(name));
        }
    }
    ;
    /**
     * Create or return the underlying object this model represents.
     */
    createObject() {
        // call constructor method overridden by every class
        let objPromise = Promise.resolve(this.constructObject());
        return objPromise.then(this.processNewObj.bind(this));
    }
    /**
     * Process a new underlying object to represent this model.
     *
     * The base implementation sets up mapping between the model,
     * the cache, and the widget manager.
     */
    processNewObj(obj) {
        obj.ipymodelId = this.model_id; // brand that sucker
        obj.ipymodel = this;
        this.obj = obj;
        return obj;
    }
    /**
     * Set up any event listeners.
     *
     * Called after object initialization is complete.
     */
    setupListeners() {
        // Handle changes in child model instance props
        for (let propName of this.childModelProperties) {
            // register listener for current child value
            var curValue = this.get(propName);
            if (curValue) {
                this.listenTo(curValue, 'change', this.onChildChanged.bind(this));
                this.listenTo(curValue, 'childchange', this.onChildChanged.bind(this));
            }
            // make sure to (un)hook listeners when child points to new object
            this.on('change:' + propName, (model, value, options) => {
                const prevModel = this.previous(propName);
                const currModel = value;
                if (prevModel) {
                    this.stopListening(prevModel);
                }
                if (currModel) {
                    this.listenTo(currModel, 'change', this.onChildChanged.bind(this));
                    this.listenTo(currModel, 'childchange', this.onChildChanged.bind(this));
                }
            }, this);
        }
        ;
        // Handle changes in data widgets/union properties
        for (let propName of this.datawidgetProperties) {
            (0, jupyter_dataserializers_1.listenToUnion)(this, propName, this.onChildChanged.bind(this), false);
        }
        ;
        this.on('change', this.onChange, this);
        this.on('msg:custom', this.onCustomMessage, this);
    }
    onChange(model, options) {
        if (options !== 'pushFromObject') {
            this.syncToObject();
        }
    }
    onChildChanged(model, options) {
        // Propagate up hierarchy:
        this.trigger('childchange', this);
    }
    onCustomMessage(content, buffers) {
    }
}
exports.ScaleModel = ScaleModel;
ScaleModel.serializers = base_1.WidgetModel.serializers;
ScaleModel.model_module = version_1.MODULE_NAME;
ScaleModel.model_module_version = version_1.MODULE_VERSION;
ScaleModel.view_name = null;
ScaleModel.view_module = null;
ScaleModel.view_module_version = version_1.MODULE_VERSION;
/**
 * A widget model of a sequential scale
 */
class SequentialScaleModel extends ScaleModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { domain: [0, 1], clamp: false });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('domain', 'clamp');
    }
}
exports.SequentialScaleModel = SequentialScaleModel;
SequentialScaleModel.serializers = Object.assign({}, ScaleModel.serializers);
/**
 * A widget model of a quantize scale
 */
class QuantizeScaleModel extends ScaleModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { domain: [0, 1], range: [0, 1] });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('domain', 'range');
    }
    constructObject() {
        return (0, d3_scale_1.scaleQuantize)();
    }
}
exports.QuantizeScaleModel = QuantizeScaleModel;
QuantizeScaleModel.serializers = Object.assign({}, ScaleModel.serializers);
QuantizeScaleModel.model_name = 'QuantizeScaleModel';
/**
 * A widget model of a quantile scale
 */
class QuantileScaleModel extends ScaleModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { domain: [0], range: [0] });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('domain', 'range');
    }
    constructObject() {
        return (0, d3_scale_1.scaleQuantile)();
    }
}
exports.QuantileScaleModel = QuantileScaleModel;
QuantileScaleModel.serializers = Object.assign({}, ScaleModel.serializers);
QuantileScaleModel.model_name = 'QuantileScaleModel';
/**
 * A widget model of a teshold scale
 */
class TresholdScaleModel extends ScaleModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { domain: [], range: [0] });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('domain', 'range');
    }
    constructObject() {
        return (0, d3_scale_1.scaleQuantile)();
    }
}
exports.TresholdScaleModel = TresholdScaleModel;
TresholdScaleModel.serializers = Object.assign({}, ScaleModel.serializers);
TresholdScaleModel.model_name = 'TresholdScaleModel';
/**
 * A widget model of an ordinal scale
 */
class OrdinalScaleModel extends ScaleModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { domain: [], range: [], unknown: d3_scale_1.scaleImplicit });
    }
    createPropertiesArrays() {
        super.createPropertiesArrays();
        this.simpleProperties.push('range', 'unknown');
    }
    constructObject() {
        return (0, d3_scale_1.scaleOrdinal)();
    }
    /**
     * Update the model attributes from the objects properties.
     */
    syncToModel(toSet) {
        toSet['domain'] = this.obj.domain();
        super.syncToModel(toSet);
    }
    /**
     * Update the model attributes from the objects properties.
     */
    syncToObject() {
        var _a;
        super.syncToObject();
        this.obj.domain((_a = this.get('domain')) !== null && _a !== void 0 ? _a : []);
    }
    ;
}
exports.OrdinalScaleModel = OrdinalScaleModel;
OrdinalScaleModel.serializers = Object.assign(Object.assign({}, ScaleModel.serializers), { unknown: {
        deserialize: (value, manager) => {
            return value === '__implicit'
                ? d3_scale_1.scaleImplicit
                : value === null
                    ? undefined
                    : value;
        },
        serialize: (value, widget) => {
            return value === d3_scale_1.scaleImplicit
                ? '__implicit'
                : value === undefined
                    ? null
                    : value;
        },
    } });
OrdinalScaleModel.model_name = 'OrdinalScaleModel';
//# sourceMappingURL=scale.js.map